﻿using System;
using System.ComponentModel;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Input;
using System.Windows.Media;

namespace TestProject
{
    /// <summary>
    /// 멀티 리스트 박스
    /// </summary>
    public partial class MultiListBox : ListBox
    {
        //////////////////////////////////////////////////////////////////////////////////////////////////// Event
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 값들 변경시 - ValuesChanged

        /// <summary>
        /// 값들 변경시
        /// </summary>
        public event EventHandler ValuesChanged;

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Field
        ////////////////////////////////////////////////////////////////////////////////////////// Static
        //////////////////////////////////////////////////////////////////////////////// Public

        #region Field

        /// <summary>
        /// 값들 의존 속성
        /// </summary>
        public static readonly DependencyProperty ValuesProperty;

        /// <summary>
        /// 텍스트들 의존 속성
        /// </summary>
        public static readonly DependencyProperty TextsProperty;

        #endregion

        ////////////////////////////////////////////////////////////////////////////////////////// Instance
        //////////////////////////////////////////////////////////////////////////////// Private

        #region Field

        /// <summary>
        /// CheckBox_Checked 이벤트 핸들러 미실행 여부
        /// </summary>
        private bool noWorking = false;

        /// <summary>
        /// 값들 내부 설정 여부
        /// </summary>
        private bool isValuesInternalSetting = false;

        /// <summary>
        /// 시작 드래그 인덱스
        /// </summary>
        private int startDragIndex = -1;

        /// <summary>
        /// 종료 드래그 인덱스
        /// </summary>
        private int endDragIndex = -1;

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Property
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 값들 - Values

        /// <summary>
        /// 값들
        /// </summary>
        public string Values
        {
            get
            {
                return (string)GetValue(ValuesProperty);
            }
            set
            {
                SetValue(ValuesProperty, value);
            }
        }

        #endregion
        #region 텍스트들 - Texts

        /// <summary>
        /// 텍스트들
        /// </summary>
        public string Texts
        {
            get
            {
                return (string)GetValue(TextsProperty);
            }
            set
            {
                SetValue(TextsProperty, value);
            }
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Constructor
        ////////////////////////////////////////////////////////////////////////////////////////// Static
        
        #region 생성자 - MultiListBox()

        /// <summary>
        /// 생성자
        /// </summary>
        static MultiListBox()
        {
            ValuesProperty = UIHelper.RegisterDependencyProperty(string.Empty, false, false, false, true, valuesPropertyChangedCallback, "Values", typeof(string), typeof(MultiListBox), null);
            TextsProperty  = UIHelper.RegisterDependencyProperty(string.Empty, false, false, false, true, null                         , "Texts" , typeof(string), typeof(MultiListBox), null);
        }

        #endregion

        ////////////////////////////////////////////////////////////////////////////////////////// Instance
        //////////////////////////////////////////////////////////////////////////////// Public

        #region 생성자 - MultiListBox()

        /// <summary>
        /// 생성자
        /// </summary>
        public MultiListBox()
        {
            InitializeComponent();


            AddCheckBoxHandler();


            AddHandler(ListBox.PreviewMouseDownEvent, new MouseButtonEventHandler(ListBox_PreviewMouseDown));
            AddHandler(ListBox.PreviewMouseUpEvent  , new MouseButtonEventHandler(ListBox_PreviewMouseUp  ));
            AddHandler(ListBox.PreviewKeyDownEvent  , new KeyEventHandler        (ListBox_PreviewKeyDown  ));

            DependencyPropertyDescriptor dependencyPropertyDescriptor = DependencyPropertyDescriptor.FromProperty(ItemsControl.ItemsSourceProperty, typeof(ListBox));

            if(dependencyPropertyDescriptor != null)
            {
                dependencyPropertyDescriptor.AddValueChanged(this, itemsSourcePropertyChanged);
            }
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Method
        ////////////////////////////////////////////////////////////////////////////////////////// Static
        //////////////////////////////////////////////////////////////////////////////// Private

        #region 값들 속성 변경시 콜백 처리하기 - valuesPropertyChangedCallback(dependencyObject, e)

        /// <summary>
        /// 값들 속성 변경시 콜백 처리하기
        /// </summary>
        /// <param name="dependencyObject">의존 객체</param>
        /// <param name="e">이벤트 인자</param>
        private static void valuesPropertyChangedCallback(DependencyObject dependencyObject, DependencyPropertyChangedEventArgs e)
        {
            MultiListBox multiListBox = dependencyObject as MultiListBox;


            if(multiListBox.noWorking) //★★★
            {
                return;
            }


            if(!multiListBox.isValuesInternalSetting)
            {
                string newValue = (e.NewValue) == null ? string.Empty : e.NewValue.ToString();

                multiListBox.SetChecked(newValue);
            }

            multiListBox.isValuesInternalSetting = false;


            string texts = multiListBox.GetTexts();

            multiListBox.Texts = texts;


            multiListBox.FireValuesChanged();
        }

        #endregion

        ////////////////////////////////////////////////////////////////////////////////////////// Instance
        //////////////////////////////////////////////////////////////////////////////// Public

        #region 체크 설정하기 - SetChecked(values)

        /// <summary>
        /// 체크 설정하기
        /// </summary>
        /// <param name="values">값 목록</param>
        public void SetChecked(string values)
        {
            try
            {
                this.noWorking = true; //★★★


                RemoveCheckBoxHandler();


                ObservableCollection<ListItem> collection = ItemsSource as ObservableCollection<ListItem>;

                if(collection == null || collection.Count == 0)
                {
                    return;
                }


                foreach(ListItem item in collection)
                {
                    item.IsChecked = false;
                }


                if(string.IsNullOrEmpty(values))
                {
                    return;
                }


                string[] valueArray = values.Split(',');
                int      valueCount = valueArray.Length;

                for(int i = valueCount - 1; i > -1; i--)
                {
                    for(int j = 0; j < collection.Count; j++)
                    {
                        ListItem item = collection[j];

                        if(item.Value.Equals(valueArray[i]))
                        {
                            item.IsChecked = true;

                            if(!this.isValuesInternalSetting) //★★★
                            {
                                int itemIndex = collection.IndexOf(item);

                                collection.Move(itemIndex, 0);
                            }

                            break;
                        }
                    }
                }
            }
            finally
            {
                this.isValuesInternalSetting = false; //★★★

                this.noWorking = false; //★★★

                AddCheckBoxHandler();
            }
        }

        #endregion
        #region 값들 구하기 - GetValues()

        /// <summary>
        /// 값들 구하기
        /// </summary>
        /// <returns>값들</returns>
        public string GetValues()
        {
            ObservableCollection<ListItem> collection = ItemsSource as ObservableCollection<ListItem>;

            if(collection == null || collection.Count == 0)
            {
                return string.Empty;
            }


            StringBuilder stringBuilder = new StringBuilder();

            foreach(ListItem item in collection)
            {
                if(item.IsChecked)
                {
                    stringBuilder.AppendFormat("{0},", item.Value);
                }
            }


            return stringBuilder.ToString().Trim().TrimEnd(',');
        }

        #endregion
        #region 텍스트들 구하기 - GetTexts()

        /// <summary>
        /// 텍스트들 구하기
        /// </summary>
        /// <returns>텍스트들</returns>
        public string GetTexts()
        {
            ObservableCollection<ListItem> collection = ItemsSource as ObservableCollection<ListItem>;

            if(collection == null || collection.Count == 0)
            {
                return string.Empty;
            }


            StringBuilder stringBuilder = new StringBuilder();

            foreach(ListItem item in collection)
            {
                if(item.IsChecked)
                {
                    stringBuilder.AppendFormat("{0},", item.Text);
                }
            }


            return stringBuilder.ToString().Trim().TrimEnd(',');
        }

        #endregion
        #region 값 항목 리스트 구하기 - GetValueItemList()

        /// <summary>
        /// 값 항목 리스트 구하기
        /// </summary>
        /// <returns>값 항목 리스트</returns>
        public List<ListItem> GetValueItemList()
        {
            ObservableCollection<ListItem> collection = ItemsSource as ObservableCollection<ListItem>;

            List<ListItem> list = new List<ListItem>();

            if(collection == null || collection.Count == 0)
            {
                return list;
            }


            foreach(ListItem item in collection)
            {
                if(item.IsChecked)
                {
                    list.Add(item);
                }
            }

            return list;
        }

        #endregion
        #region 모든 항목 체크하기 - CheckAll()

        /// <summary>
        /// 모든 항목 체크하기
        /// </summary>
        public void CheckAll()
        {
            try
            {
                this.noWorking = true; //★★★


                RemoveCheckBoxHandler();


                ObservableCollection<ListItem> collection = ItemsSource as ObservableCollection<ListItem>;

                if(collection == null || collection.Count == 0)
                {
                    return;
                }


                foreach(ListItem item in collection)
                {
                    item.IsChecked = true;
                }


                this.noWorking = false; //★★★


                string values = GetValues();

                if(Values != values)
                {
                    this.isValuesInternalSetting = true; //★★★

                    Values = values;
                }
            }
            finally
            {
                AddCheckBoxHandler();
            }
        }

        #endregion
        #region 모든 항목 체크 취소하기 - UncheckAll()

        /// <summary>
        /// 모든 항목 체크 취소하기
        /// </summary>
        public void UncheckAll()
        {
            try
            {
                this.noWorking = true; //★★★


                RemoveCheckBoxHandler();


                ObservableCollection<ListItem> collection = ItemsSource as ObservableCollection<ListItem>;

                if(collection == null || collection.Count == 0)
                {
                    return;
                }


                foreach(ListItem item in collection)
                {
                    item.IsChecked = false;
                }


                this.noWorking = false; //★★★


                string values = GetValues();

                if(Values != values)
                {
                    this.isValuesInternalSetting = true; //★★★

                    Values = values;
                }
            }
            finally
            {
                AddCheckBoxHandler();
            }
        }

        #endregion
        #region 체크 항목 가장 위쪽으로 이동시키기 - MoveTopMost()

        /// <summary>
        /// 체크 항목 가장 위쪽으로 이동시키기
        /// </summary>
        public void MoveTopMost()
        {
            ObservableCollection<ListItem> collection = ItemsSource as ObservableCollection<ListItem>;

            if(collection == null || collection.Count == 0)
            {
                return;
            }


            SelectedItems.Clear();


            List<ListItem> list = GetValueItemList();

            foreach(ListItem item in list)
            {
                collection.Remove(item);
            }


            int index = 0;

            foreach(ListItem item in list)
            {
                collection.Insert(index++, item);

                SelectedItems.Add(item);
            }
        }

        #endregion
        #region 선택 항목 위쪽으로 이동시키기 - MoveTop()

        /// <summary>
        /// 선택 항목 위쪽으로 이동시키기
        /// </summary>
        public void MoveTop()
        {
            ObservableCollection<ListItem> collection = ItemsSource as ObservableCollection<ListItem>;

            if(collection == null || collection.Count == 0)
            {
                return;
            }


            if(SelectedItems.Count == 0)
            {
                return;
            }


            List<ListItem> list = new List<ListItem>();

            foreach(ListItem item in SelectedItems)
            {
                list.Add(item);
            }


            int insertIndex = Math.Max(collection.IndexOf(list[0]) - 1, 0);

            foreach(ListItem item in list)
            {
                collection.Remove(item);
            }


            foreach(ListItem item in list)
            {
                collection.Insert(insertIndex++, item);

                SelectedItems.Add(item);
            }
            

            string values = GetValues();

            if(Values != values)
            {
                this.isValuesInternalSetting = true; //★★★

                Values = values;
            }
        }

        #endregion
        #region 선택 항목 아래쪽으로 이동시키기 - MoveBottom()

        /// <summary>
        /// 선택 항목 아래쪽으로 이동시키기
        /// </summary>
        public void MoveBottom()
        {
            ObservableCollection<ListItem> collection = ItemsSource as ObservableCollection<ListItem>;

            if(collection == null || collection.Count == 0)
            {
                return;
            }


            if(SelectedItems.Count == 0)
            {
                return;
            }


            List<ListItem> list = new List<ListItem>();

            foreach(ListItem item in SelectedItems)
            {
                list.Add(item);
            }


            int lastSelectedItemIndex = collection.IndexOf(list[list.Count - 1]);

            if(lastSelectedItemIndex == collection.Count - 1)
            {
                foreach(ListItem item in list)
                {
                    collection.Remove(item);
                }

                foreach(ListItem item in list)
                {
                    collection.Add(item);

                    SelectedItems.Add(item);
                }
            }
            else
            {
                int baseItemIndex = Math.Min(lastSelectedItemIndex + 1, collection.Count - 1);

                ListItem baseItem = collection[baseItemIndex] as ListItem;

                foreach(ListItem item in list)
                {
                    collection.Remove(item);
                }

                int insertIndex = collection.IndexOf(baseItem) + 1;

                if(insertIndex > collection.Count - 1)
                {
                    foreach(ListItem item in list)
                    {
                        collection.Add(item);

                        SelectedItems.Add(item);
                    }
                }
                else
                {
                    foreach(ListItem item in list)
                    {
                        collection.Insert(insertIndex++, item);

                        SelectedItems.Add(item);
                    }
                }
            }


            string values = GetValues();

            if(Values != values)
            {
                this.isValuesInternalSetting = true; //★★★

                Values = values;
            }
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////// Protected

        #region 값들 변경시 이벤트 발생시키기 - FireValuesChanged()

        /// <summary>
        /// 값들 변경시 이벤트 발생시키기
        /// </summary>
        protected void FireValuesChanged()
        {
            if(ValuesChanged != null)
            {
                ValuesChanged(this, EventArgs.Empty);
            }
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////// Private
        ////////////////////////////////////////////////////////////////////// Event

        #region 항목 소스 속성 변경시 처리하기 - itemsSourcePropertyChanged(sender, e)

        /// <summary>
        /// 항목 소스 속성 변경시 처리하기
        /// </summary>
        /// <param name="sender">이벤트 발생자</param>
        /// <param name="e">이벤트 인자</param>
        private void itemsSourcePropertyChanged(object sender, EventArgs e)
        {
            Values = string.Empty;
        }

        #endregion
        #region 리스트 박스 마우스 DOWN 미리보기 처리하기 - ListBox_PreviewMouseDown(sender, e)

        /// <summary>
        /// 리스트 박스 마우스 DOWN 미리보기 처리하기
        /// </summary>
        /// <param name="sender">이벤트 발생자</param>
        /// <param name="e">이벤트 인자</param>
        private void ListBox_PreviewMouseDown(object sender, MouseButtonEventArgs e)
        {
            if(e.OriginalSource.GetType() != typeof(TextBlock) && e.OriginalSource.GetType() != typeof(Image))
            {
                return;
            }


            HitTestResult result = VisualTreeHelper.HitTest(this, e.GetPosition(this));


            ListBoxItem listBoxItem = UIHelper.FindParent<ListBoxItem>(result.VisualHit);

            if(listBoxItem == null)
            {
                return;
            }


            ListItem listItem = listBoxItem.Content as ListItem;

            if(listItem == null)
            {
                return;
            }


            ObservableCollection<ListItem> collection = ItemsSource as ObservableCollection<ListItem>;

            if(collection == null || collection.Count == 0)
            {
                return;
            }


            this.startDragIndex = collection.IndexOf(listItem);
        }

        #endregion
        #region 리스트 박스 마우스 UP 미리보기 처리하기 - ListBox_PreviewMouseUp(sender, e)

        /// <summary>
        /// 리스트 박스 마우스 UP 미리보기 처리하기
        /// </summary>
        /// <param name="sender">이벤트 발생자</param>
        /// <param name="e">이벤트 인자</param>
        private void ListBox_PreviewMouseUp(object sender, MouseButtonEventArgs e)
        {
            if(e.OriginalSource.GetType() != typeof(TextBlock) && e.OriginalSource.GetType() != typeof(Image))
            {
                return;
            }


            HitTestResult result = VisualTreeHelper.HitTest(this, e.GetPosition(this));


            ListBoxItem listBoxItem = UIHelper.FindParent<ListBoxItem>(result.VisualHit);

            if(listBoxItem == null)
            {
                return;
            }


            ListItem listItem = listBoxItem.Content as ListItem;

            if(listItem == null)
            {
                return;
            }


            ObservableCollection<ListItem> collection = ItemsSource as ObservableCollection<ListItem>;

            if(collection == null || collection.Count == 0)
            {
                return;
            }


            this.endDragIndex = collection.IndexOf(listItem);


            if(this.startDragIndex != -1 && this.endDragIndex != -1)
            {
                if(this.startDragIndex > this.endDragIndex)
                {
                    int temporaryIndex = this.startDragIndex;

                    this.startDragIndex = this.endDragIndex;

                    this.endDragIndex = temporaryIndex;
                }

                try
                {
                    RemoveCheckBoxHandler();


                    for(int i = this.startDragIndex; i < this.endDragIndex + 1; i++)
                    {
                        collection[i].IsChecked = !collection[i].IsChecked;
                    }


                    string values = GetValues();

                    if(Values != values)
                    {
                        this.isValuesInternalSetting = true; //★★★

                        Values = values;
                    }
                }
                finally
                {
                    AddCheckBoxHandler();
                }
            }
        }

        #endregion
        #region 리스트 박스 키 DOWN 미리보기 처리하기 - ListBox_PreviewKeyDown(sender, e)

        /// <summary>
        /// 리스트 박스 키 DOWN 미리보기 처리하기
        /// </summary>
        /// <param name="sender">이벤트 발생자</param>
        /// <param name="e">이벤트 인자</param>
        private void ListBox_PreviewKeyDown(object sender, KeyEventArgs e)
        {
            if(e.Key == Key.Space)
            {
                if(SelectedItem != null)
                {
                    ListItem item = SelectedItem as ListItem;

                    if(item != null)
                    {
                        item.IsChecked = !item.IsChecked;
                    }

                    e.Handled = true;
                }
            }
        }

        #endregion
        #region 체크 박스 체크시 처리하기 - CheckBox_Checked(sender, e)

        /// <summary>
        /// 체크 박스 체크시 처리하기
        /// </summary>
        /// <param name="sender">이벤트 발생자</param>
        /// <param name="e">이벤트 인자</param>
        private void CheckBox_Checked(object sender, RoutedEventArgs e)
        {
            if(this.noWorking)
            {
                return;
            }


            CheckBox checkBox = e.OriginalSource as CheckBox;

            if(checkBox != null)
            {
                string values = GetValues();

                if(Values != values)
                {
                    this.isValuesInternalSetting = true; //★★★

                    Values = values;
                }

                return;
            }
        }

        #endregion

        ////////////////////////////////////////////////////////////////////// Function

        #region 체크 박스 핸들러 추가하기 - AddCheckBoxHandler()

        /// <summary>
        /// 체크 박스 핸들러 추가하기
        /// </summary>
        private void AddCheckBoxHandler()
        {
            AddHandler(CheckBox.CheckedEvent  , new RoutedEventHandler(CheckBox_Checked));
            AddHandler(CheckBox.UncheckedEvent, new RoutedEventHandler(CheckBox_Checked));
        }

        #endregion
        #region 체크 박스 핸들러 제거하기 - RemoveCheckBoxHandler()

        /// <summary>
        /// 체크 박스 핸들러 제거하기
        /// </summary>
        private void RemoveCheckBoxHandler()
        {
            RemoveHandler(CheckBox.CheckedEvent  , new RoutedEventHandler(CheckBox_Checked));
            RemoveHandler(CheckBox.UncheckedEvent, new RoutedEventHandler(CheckBox_Checked));
        }

        #endregion
    }
}