﻿using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Linq;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Markup;
using System.Windows.Media;

namespace TestProject
{
    /// <summary>
    /// UI 헬퍼
    /// </summary>
    public class UIHelper
    {
        //////////////////////////////////////////////////////////////////////////////////////////////////// Method
        ////////////////////////////////////////////////////////////////////////////////////////// Static
        //////////////////////////////////////////////////////////////////////////////// Public

        #region 의존 속성 등록하기 - RegisterDependencyProperty(defaultValue, affectsRender, affectsMeasure, affectsArrange, inherits, propertyChangedCallback, propertyName, propertyType, ownerType, validateValueCallback)

        /// <summary>
        /// 의존 속성 등록하기
        /// </summary>
        /// <param name="defaultValue">디폴트 값</param>
        /// <param name="affectsRender">렌더링 여향 여부</param>
        /// <param name="affectsMeasure">크기 측정 영향 여부</param>
        /// <param name="affectsArrange">배치 영향 여부</param>
        /// <param name="inherits">상속 여부</param>
        /// <param name="propertyChangedCallback">속성 변경 콜백</param>
        /// <param name="propertyName">속성명</param>
        /// <param name="propertyType">속성 타입</param>
        /// <param name="ownerType">소유자 타입</param>
        /// <param name="validateValueCallback">값 무결성 콜백</param>
        /// <returns>의존 속성</returns>
        public static DependencyProperty RegisterDependencyProperty(object defaultValue, bool affectsRender, bool affectsMeasure, bool affectsArrange, bool inherits, PropertyChangedCallback propertyChangedCallback, string propertyName, Type propertyType, Type ownerType, ValidateValueCallback validateValueCallback)
        {
            FrameworkPropertyMetadata frameworkPropertyMetadata = new FrameworkPropertyMetadata();

            frameworkPropertyMetadata.DefaultValue   = defaultValue;
            frameworkPropertyMetadata.AffectsRender  = affectsRender;
            frameworkPropertyMetadata.AffectsMeasure = affectsMeasure;
            frameworkPropertyMetadata.AffectsArrange = affectsArrange;
            frameworkPropertyMetadata.Inherits       = inherits;

            frameworkPropertyMetadata.PropertyChangedCallback += propertyChangedCallback;

            return DependencyProperty.Register(propertyName, propertyType, ownerType, frameworkPropertyMetadata, validateValueCallback);
        }

        #endregion
        #region 첨부 속성 등록하기 - RegisterAttachedProperty(defaultValue, propertyChangedCallback, propertyName, propertyType, ownerType, validateValueCallback)

        /// <summary>
        /// 첨부 속성 등록하기
        /// </summary>
        /// <param name="defaultValue">디폴트 값</param>
        /// <param name="propertyChangedCallback">속성 변경 콜백</param>
        /// <param name="propertyName">속성명</param>
        /// <param name="propertyType">속성 타입</param>
        /// <param name="ownerType">소유자 타입</param>
        /// <param name="validateValueCallback">값 무결성 콜백</param>
        /// <returns>첨부 속성</returns>
        public static DependencyProperty RegisterAttachedProperty(object defaultValue, PropertyChangedCallback propertyChangedCallback, string propertyName, Type propertyType, Type ownerType, ValidateValueCallback validateValueCallback)
        {
            FrameworkPropertyMetadata frameworkPropertyMetadata = new FrameworkPropertyMetadata();

            frameworkPropertyMetadata.DefaultValue = defaultValue;

            frameworkPropertyMetadata.PropertyChangedCallback += propertyChangedCallback;

            return DependencyProperty.RegisterAttached(propertyName, propertyType, ownerType, frameworkPropertyMetadata, validateValueCallback);
        }
        
        #endregion
        #region 바인딩 설정하기 - SetBinding(targetElement, targetProperty, source, sourceProperty, bindingMode, updateSourceTrigger)

        /// <summary>
        /// 바인딩 설정하기
        /// </summary>
        /// <param name="targetElement">타겟 엘리먼트</param>
        /// <param name="targetProperty">타겟 의존 속성</param>
        /// <param name="source">소스 객체</param>
        /// <param name="sourceProperty">소스 의존 속성</param>
        /// <param name="bindingMode">바인딩 모드</param>
        /// <param name="updateSourceTrigger">소스 업데이트 트리거</param>
        public static void SetBinding(FrameworkElement targetElement, DependencyProperty targetProperty, object source, DependencyProperty sourceProperty, BindingMode bindingMode, UpdateSourceTrigger updateSourceTrigger)
        {
            Binding binding = new Binding();

            binding.Source              = source;
            binding.Path                = new PropertyPath(sourceProperty);
            binding.Mode                = bindingMode;
            binding.UpdateSourceTrigger = updateSourceTrigger;

            targetElement.SetBinding(targetProperty, binding);
        }

        #endregion
        #region 체크 라디오 버튼 구하기 - GetCheckedRadioButton(groupBox)

        /// <summary>
        /// 체크 라디오 버튼 구하기
        /// </summary>
        /// <param name="groupBox">그룹 박스</param>
        /// <returns>체크 라디오 버튼</returns>
        public static RadioButton GetCheckedRadioButton(GroupBox groupBox)
        {
            Panel panel = groupBox.Content as Panel;

            if(panel != null)
            {
                foreach(UIElement child in panel.Children)
                {
                    RadioButton radioButton = child as RadioButton;

                    if(radioButton != null && radioButton.IsChecked.GetValueOrDefault())
                    {
                        return radioButton;
                    }
                }
            }

            return null;
        }

        #endregion
        #region 부모 찾기 - FindParent<TParent>(dependencyObject)

        /// <summary>
        /// 부모 찾기
        /// </summary>
        /// <typeparam name="TParent">부모 타입</typeparam>
        /// <param name="dependencyObject">의존 객체</param>
        /// <returns>부모</returns>
        public static TParent FindParent<TParent>(DependencyObject dependencyObject) where TParent : class
        {
            TParent parent = null;

            DependencyObject parentDependencyObject = VisualTreeHelper.GetParent(dependencyObject);

            if(parentDependencyObject is TParent)
            {
                parent = parentDependencyObject as TParent;
            }
            else if(parentDependencyObject != null)
            {
                parent = FindParent<TParent>(parentDependencyObject);
            }

            return parent;
        }

        #endregion
        #region 폰트명 구하기 - GetFontName(fontFamily)

        /// <summary>
        /// 폰트명 구하기
        /// </summary>
        /// <param name="fontFamily">폰트 패밀리</param>
        /// <returns>폰트명</returns>
        public static string GetFontName(FontFamily fontFamily)
        {
            if(fontFamily == null)
            {
                return string.Empty;
            }

            if(fontFamily.FamilyNames.Count > 1 && fontFamily.FamilyNames.Keys.ElementAt<XmlLanguage>(fontFamily.FamilyNames.Count - 1).IetfLanguageTag == "ko-kr")
            {
                return fontFamily.FamilyNames.Values.ElementAt<string>(fontFamily.FamilyNames.Count - 1);
            }
            else
            {
                return fontFamily.FamilyNames.Values.ElementAt<string>(0);
            }
        }

        #endregion

        #region 폰트 패밀리 데이터 구하기 - GetFontFamilyData()

        /// <summary>
        /// 폰트 패밀리 데이터 구하기
        /// </summary>
        /// <returns>폰트 패밀리 데이터</returns>
        public static ObservableCollection<ListItem> GetFontFamilyData()
        {
            SortedList<string, FontFamily> sortedList = new SortedList<string, FontFamily>();

            foreach(FontFamily fontFamily in Fonts.SystemFontFamilies)
            {
                string fontName = UIHelper.GetFontName(fontFamily);

                sortedList.Add(fontName, fontFamily);
            }

            ObservableCollection<ListItem> collection = new ObservableCollection<ListItem>();

            for(int i = 0; i < sortedList.Keys.Count; i++)
            {
                ListItem item = new ListItem(sortedList.Keys[i]);

                item.Tag = sortedList[sortedList.Keys[i]];

                collection.Add(item);
            }

            return collection;
        }

        #endregion
        #region 폰트 스타일 데이터 구하기 - GetFontStyleData(fontFamily)

        /// <summary>
        /// 폰트 스타일 데이터 구하기
        /// </summary>
        /// <param name="fontFamily">폰트 패밀리</param>
        /// <returns>폰트 스타일 데이터</returns>
        public static ObservableCollection<ListItem> GetFontStyleData(FontFamily fontFamily)
        {
            List<FontStyle> list = new List<FontStyle>();

            foreach(FamilyTypeface familyTypeface in fontFamily.FamilyTypefaces)
            {
                if(!list.Contains(familyTypeface.Style))
                {
                    if(familyTypeface.Style == FontStyles.Normal)
                    {
                        list.Insert(0, familyTypeface.Style);
                    }
                    else
                    {
                        list.Add(familyTypeface.Style);
                    }
                }
            }

            ObservableCollection<ListItem> collection = new ObservableCollection<ListItem>();

            foreach(FontStyle fontStyle in list)
            {
                ListItem item = new ListItem(fontStyle.ToString());

                item.Tag = fontStyle;

                collection.Add(item);
            }

            return collection;
        }

        #endregion
        #region 폰트 웨이트 데이터 구하기 - GetFontWeightData(fontFamily)

        /// <summary>
        /// 폰트 웨이트 데이터 구하기
        /// </summary>
        /// <param name="fontFamily">폰트 패밀리</param>
        /// <returns>폰트 웨이트 데이터</returns>
        public static ObservableCollection<ListItem> GetFontWeightData(FontFamily fontFamily)
        {
            List<FontWeight> list = new List<FontWeight>();

            foreach(FamilyTypeface familyTypeface in fontFamily.FamilyTypefaces)
            {
                if(!list.Contains(familyTypeface.Weight))
                {
                    if(familyTypeface.Weight == FontWeights.Normal)
                    {
                        list.Insert(0, familyTypeface.Weight);
                    }
                    else
                    {
                        list.Add(familyTypeface.Weight);
                    }
                }
            }

            ObservableCollection<ListItem> collection = new ObservableCollection<ListItem>();

            foreach(FontWeight fontWeight in list)
            {
                ListItem item = new ListItem(fontWeight.ToString());

                item.Tag = fontWeight;

                collection.Add(item);
            }

            return collection;
        }

        #endregion
        #region 폰트 스트레치 데이터 구하기 - GetFontStretchData(fontFamily)

        /// <summary>
        /// 폰트 스트레치 데이터 구하기
        /// </summary>
        /// <param name="fontFamily">폰트 패밀리</param>
        /// <returns>폰트 스트레치 데이터</returns>
        public static ObservableCollection<ListItem> GetFontStretchData(FontFamily fontFamily)
        {
            List<FontStretch> list = new List<FontStretch>();

            foreach(FamilyTypeface familyTypeface in fontFamily.FamilyTypefaces)
            {
                if(!list.Contains(familyTypeface.Stretch))
                {
                    if(familyTypeface.Stretch == FontStretches.Normal)
                    {
                        list.Insert(0, familyTypeface.Stretch);
                    }
                    else
                    {
                        list.Add(familyTypeface.Stretch);
                    }
                }
            }

            ObservableCollection<ListItem> collection = new ObservableCollection<ListItem>();

            foreach(FontStretch fontStretch in list)
            {
                ListItem item = new ListItem(fontStretch.ToString());

                item.Tag = fontStretch;

                collection.Add(item);
            }

            return collection;
        }

        #endregion
        #region 폰트 크기 데이터 구하기 - GetFontSizeData()

        /// <summary>
        /// 폰트 크기 데이터 구하기
        /// </summary>
        /// <returns>폰트 크기 데이터</returns>
        public static ObservableCollection<ListItem> GetFontSizeData()
        {
            double[] fontSizeArray = new double[] { 8, 9, 10, 11, 12, 14, 16, 18, 20, 22, 24, 26, 28, 36, 48, 72 };

            ObservableCollection<ListItem> collection = new ObservableCollection<ListItem>();

            foreach(double fontSize in fontSizeArray)
            {
                ListItem item = new ListItem(fontSize.ToString());

                item.Tag = fontSize;

                collection.Add(item);
            }

            return collection;
        }

        #endregion
    }
}